window.Gleam.initSearch([{"doc":"themis","title":"themis","content":"# Themis\n\nPrometheus client in pure Gleam!\n\nPlease remember that Themis is still in early development.\n\nOnly Erlang target supported currently.\n\n[![Package Version](https://img.shields.io/hexpm/v/themis)](https://hex.pm/packages/themis)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/themis/)\n\n```sh\ngleam add themis\n```\n\n## Quick Start\n\n```gleam\nimport gleam/dict\nimport gleam/io\nimport gleam/set\nimport themis\nimport themis/counter\nimport themis/gauge\nimport themis/histogram\nimport themis/number\n\npub fn main() {\n  // initialize the metrics store\n  themis.init()\n\n  // Gauge\n\n  // This can fail if the metric name is invalid\n  let assert Ok(_) =\n    gauge.new(\n      &quot;my_first_metric&quot;,\n      &quot;A gauge Prometheus metric&quot;,\n    )\n\n  let labels = dict.from_list([#(&quot;foo&quot;, &quot;bar&quot;)])\n  let value = number.integer(10)\n  let assert Ok(_) =\n    gauge.observe(&quot;my_first_metric&quot;, labels, value)\n\n  // Counter\n\n  let assert Ok(_) =\n    counter.new(\n      &quot;my_second_metric&quot;,\n      &quot;A counter Prometheus metric&quot;,\n    )\n\n  let labels = dict.from_list([#(&quot;wibble&quot;, &quot;wobble&quot;)])\n  let other_labels = dict.from_list([#(&quot;wii&quot;, &quot;woo&quot;)])\n  let assert Ok(_) =\n    counter.new(&quot;my_second_metric&quot;, &quot;A counter Prometheus metric&quot;)\n  let assert Ok(_) =\n    counter.increment(&quot;my_second_metric&quot;, labels)\n  let assert Ok(_) =\n    counter.increment_by(\n      &quot;my_second_metric&quot;,\n      other_labels,\n      number.decimal(1.2),\n    )\n\n  // Histogram\n\n  // Histograms work with buckets. Each bucket needs an upper boundary.\n  // Read more about histograms here https://prometheus.io/docs/practices/histograms/\n  let buckets =\n    set.from_list([\n      number.decimal(0.05),\n      number.decimal(0.1),\n      number.decimal(0.25),\n      number.decimal(0.5),\n      number.integer(1),\n    ])\n  let assert Ok(_) =\n    histogram.new(\n      &quot;my_third_metric&quot;,\n      &quot;A histogram Prometheus metric&quot;,\n      buckets,\n    )\n\n  let value = number.integer(20)\n  let other_value = number.decimal(1.5)\n  let labels = dict.from_list([#(&quot;toto&quot;, &quot;tata&quot;)])\n  let other_labels = dict.from_list([#(&quot;toto&quot;, &quot;titi&quot;)])\n  let assert Ok(_) =\n    histogram.observe(&quot;my_third_metric&quot;, labels, value)\n  // When incrementing a histogram with new labels, a new histogram will automatically be initialized\n  let assert Ok(_) =\n    histogram.observe(\n      &quot;my_third_metric&quot;,\n      other_labels,\n      other_value,\n    )\n\n  // Printing all the metrics as a Prometheus-scrapable String\n  let assert Ok(prometheus_string) = themis.print()\n  io.println(prometheus_string)\n}\n\n```\n\nThis code will print the following prometheus-compatible metrics:\n\n```\n# HELP my_first_metric A gauge Prometheus metric\n# TYPE my_first_metric gauge\nmy_first_metric{foo=&quot;bar&quot;} 10\n\n# HELP my_second_metric_total A counter Prometheus metric\n# TYPE my_second_metric_total counter\nmy_second_metric_total{wibble=&quot;wobble&quot;} 1\nmy_second_metric_total{wii=&quot;woo&quot;} 1.2\n\n# HELP my_third_metric A histogram Prometheus metric\n# TYPE my_third_metric histogram\nmy_third_metric_bucket{le=&quot;0.05&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.1&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.25&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;0.5&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;1&quot;,toto=&quot;tata&quot;} 0\nmy_third_metric_bucket{le=&quot;+Inf&quot;,toto=&quot;tata&quot;} 1\nmy_third_metric_sum{toto=&quot;tata&quot;} 1\nmy_third_metric_count{toto=&quot;tata&quot;} 1\n\nmy_third_metric_bucket{le=&quot;0.05&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.1&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.25&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;0.5&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;1&quot;,toto=&quot;titi&quot;} 0\nmy_third_metric_bucket{le=&quot;+Inf&quot;,toto=&quot;titi&quot;} 1\nmy_third_metric_sum{toto=&quot;titi&quot;} 1\nmy_third_metric_count{toto=&quot;titi&quot;} 1\n\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/themis&gt;.\n\n## Usage\n\n### Working with Different Numeric Types\n\nThemis metric values are set using the dedicated `Number` type. There are 5 number types available:\n\n```gleam\nimport themis/number\n\n// Integer values\nlet integer = number.integer(1_234_567)\n\n// Decimal (float) values\nlet decimal = number.decimal(23.5)\n\n// Special values\nlet positive_infinity = number.positive_infinity()\nlet negative_infinity = number.negative_infinity()\nlet not_a_number = number.not_a_number()\n```\n### Metric Types\n\n#### Gauges\n\nGauges are metrics that represent a single numerical value that can arbitrarily go up and down. They are typically used for measured values like temperatures, current memory usage, or number of active connections.\n\n```gleam\nimport themis/gauge\n\n// Create a new gauge metric\nlet assert Ok(_) = \n  gauge.new(\n    &quot;process_memory_bytes&quot;,\n    &quot;Current memory usage in bytes&quot;,\n  )\n\n// Set a gauge value with labels\nlet labels = dict.from_list([#(&quot;process&quot;, &quot;web_server&quot;)])\nlet value = number.integer(52_428_800)  // 50MB in bytes\nlet assert Ok(_) =\n  gauge.observe(&quot;process_memory_bytes&quot;, labels, value)\n```\n\n#### Counters\n\nCounters are cumulative metrics that can only increase or be reset to zero. They are typically used to count requests served, tasks completed, errors occurred, or other countable occurrences.\n\n```gleam\nimport themis/counter\n\n// Create a new counter metric\nlet assert Ok(_) =\n  counter.new(\n    &quot;http_requests_total&quot;,\n    &quot;Total number of HTTP requests made&quot;,\n  )\n\n// Currently, counters cannot be initialized.\n// To have a counter of value 0, you must increment it by 0:\nlet labels = dict.from_list([#(&quot;method&quot;, &quot;GET&quot;), #(&quot;path&quot;, &quot;/api/users&quot;)])\nlet assert Ok(_) =\n  counter.increment_by(&quot;http_requests_total&quot;, labels, number.integer(0))\n\n// Increment counter by 1\nlet assert Ok(_) =\n  counter.increment(&quot;http_requests_total&quot;, labels)\n\n// Increment counter by specific amount\nlet assert Ok(_) =\n  counter.increment_by(\n    &quot;http_requests_total&quot;,\n    labels,\n    number.decimal(5.0),\n  )\n```\n\n#### Histograms\n\nHistograms sample observations (usually duration or response size) and count them in configurable buckets. They also provide a sum of all observed values and a count of observations.\n\n```gleam\nimport themis/histogram\n\n// Define histogram buckets (upper bounds of observation buckets in seconds)\nlet buckets =\n  set.from_list([\n    number.decimal(0.005),  // 5ms\n    number.decimal(0.01),   // 10ms\n    number.decimal(0.025),  // 25ms\n    number.decimal(0.05),   // 50ms\n    number.decimal(0.1),    // 100ms\n    number.decimal(0.25),   // 250ms\n    number.decimal(0.5),    // 500ms\n    number.decimal(1.0),    // 1s\n  ])\n\n// Create a new histogram metric\nlet assert Ok(_) =\n  histogram.new(\n    &quot;http_request_duration_seconds&quot;,\n    &quot;HTTP request duration in seconds&quot;,\n    buckets,\n  )\n\n// Record an observation\nlet labels = dict.from_list([#(&quot;method&quot;, &quot;POST&quot;), #(&quot;path&quot;, &quot;/api/users&quot;)])\nlet duration = number.decimal(0.157)  // 157ms\nlet assert Ok(_) =\n  histogram.observe(\n    &quot;http_request_duration_seconds&quot;,\n    labels,\n    duration,\n  )\n```\n\nEach histogram observation is counted in all buckets with upper bounds greater than the observation value. The `+Inf` bucket is automatically added and counts all observations. Additionally, histograms track the sum of all observed values and the total count of observations.\n\n#### Summaries\n\nSummaries have not yet been implemented, because at first glance it seems an accurate summary must keep a complete history of all the observed values, which will be a huge memory hog. This means I would have to implement some algorithm that goes way above my head to instead derive an approximation. I ain&#39;t doin&#39; that (maybe one day if I need summaries but don&#39;t hold your breath).&lt;br&gt;\nIf you&#39;re feeling adventurous, feel free to open a PR.\n\n\n## Known issues\n- Javascript target is not supported. While it is not *impossible* to implement, I am not a Javascript developer. To make Themis compatible with the Javascript target, a replacement for the Erlang ETS tables (which store all the metrics) must be used. If you&#39;re interested in implementing a Javascript-compatible metrics store for Themis, please open an issue.\n\n## License\n\nMIT","url":"index.html"},{"doc":"themis/number","title":"ComparisonError","content":"pub type ComparisonError {\n  NaNValue\n}\n\nNaNValue\n\n\nSynonyms:\nthemis/number.ComparisonError\nthemis/number ComparisonError","url":"themis/number.html#ComparisonError"},{"doc":"themis/number","title":"Number","content":"pub type Number {\n  PosInf\n  NegInf\n  NaN\n  Dec(Float)\n  Int(Int)\n}\n Prometheus numbers.\n Helpful for displaying and adding/comparing values\n of different types.\nPosInf\n\n\nNegInf\n\n\nNaN\n\n\nDec(Float)\n\n\nInt(Int)\n\n\nSynonyms:\nthemis/number.Number\nthemis/number Number","url":"themis/number.html#Number"},{"doc":"themis/number","title":"add","content":"pub fn add(value1 val1: Number, value2 val2: Number) -&gt; Number\n Add two numbers.\n Any `NaN` input value will always return `NaN`.\n `PosInf` + `NegInf` = `NaN`\nSynonyms:\nthemis/number.add\nthemis/number add","url":"themis/number.html#add"},{"doc":"themis/number","title":"compare","content":"pub fn compare(\n  to val1: Number,\n  compare val2: Number,\n) -&gt; Result(Order, ComparisonError)\n Compare two numbers (see gleam_stdlib/order)\n Will return an error if either value is NaN\nSynonyms:\nthemis/number.compare\nthemis/number compare","url":"themis/number.html#compare"},{"doc":"themis/number","title":"decimal","content":"pub fn decimal(value value: Float) -&gt; Number\n Creates a Number representing a decimal value.\nSynonyms:\nthemis/number.decimal\nthemis/number decimal","url":"themis/number.html#decimal"},{"doc":"themis/number","title":"integer","content":"pub fn integer(value value: Int) -&gt; Number\n Creates a Number representing an integer value.\nSynonyms:\nthemis/number.integer\nthemis/number integer","url":"themis/number.html#integer"},{"doc":"themis/number","title":"negative_infinity","content":"pub fn negative_infinity() -&gt; Number\n Creates a Number representing negative infinity.\nSynonyms:\nthemis/number.negative_infinity\nthemis/number negative_infinity","url":"themis/number.html#negative_infinity"},{"doc":"themis/number","title":"not_a_number","content":"pub fn not_a_number() -&gt; Number\n Creates a Number representing NaN (Not a Number).\nSynonyms:\nthemis/number.not_a_number\nthemis/number not_a_number","url":"themis/number.html#not_a_number"},{"doc":"themis/number","title":"positive_infinity","content":"pub fn positive_infinity() -&gt; Number\n Creates a Number representing positive infinity.\nSynonyms:\nthemis/number.positive_infinity\nthemis/number positive_infinity","url":"themis/number.html#positive_infinity"},{"doc":"themis/number","title":"print","content":"pub fn print(number: Number) -&gt; String\n Prometheus-scrapable representation of a number\nSynonyms:\nthemis/number.print\nthemis/number print","url":"themis/number.html#print"},{"doc":"themis/number","title":"unsafe_compare","content":"pub fn unsafe_compare(\n  compare val1: Number,\n  to val2: Number,\n) -&gt; Order\n Compare two numbers (see gleam_stdlib/order)\n Will panic if either value is NaN\nSynonyms:\nthemis/number.unsafe_compare\nthemis/number unsafe_compare","url":"themis/number.html#unsafe_compare"},{"doc":"themis/number","title":"themis/number","content":"","url":"themis/number.html"},{"doc":"themis/counter","title":"CounterError","content":"pub type CounterError {\n  MetricError(metric.MetricError)\n  StoreError(store.StoreError)\n  InvalidIncrement(value: Number)\n  NegativeIncrement\n  CounterNameShouldEndWithTotal\n  LabelError(label.LabelError)\n}\n\nMetricError(metric.MetricError)\n\n\nStoreError(store.StoreError)\n\n\nInvalidIncrement(value: Number)\n\n\nNegativeIncrement\n\n\nCounterNameShouldEndWithTotal\n\n\nLabelError(label.LabelError)\n\n\nSynonyms:\nthemis/counter.CounterError\nthemis/counter CounterError","url":"themis/counter.html#CounterError"},{"doc":"themis/counter","title":"increment","content":"pub fn increment(\n  name name: String,\n  labels labels: Dict(String, String),\n) -&gt; Result(Nil, CounterError)\n Increments a counter metric for the given metric name.\n Will return an error if the name is invalid, not a registered metric\n or not of the correct metric type.\n Will return an error if any of the labels have an invalid key.\nSynonyms:\nthemis/counter.increment\nthemis/counter increment","url":"themis/counter.html#increment"},{"doc":"themis/counter","title":"increment_by","content":"pub fn increment_by(\n  name name: String,\n  labels labels: Dict(String, String),\n  value value: Number,\n) -&gt; Result(Nil, CounterError)\n Increments a counter metric for the given metric name.\n Will return an error if the name is invalid, not a registered metric\n or not of the correct metric type.\n Will return an error if any of the labels have an invalid key.\n Will return an error if the value is NaN, PosInf or NegInf.\nSynonyms:\nthemis/counter.increment_by\nthemis/counter increment_by","url":"themis/counter.html#increment_by"},{"doc":"themis/counter","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n) -&gt; Result(Nil, CounterError)\n Registers a new counter metric to the store.\n Counter metric names must end with `_total`.\n Will return an error if the metric name is invalid\n or already used by another metric.\nSynonyms:\nthemis/counter.new\nthemis/counter new","url":"themis/counter.html#new"},{"doc":"themis/counter","title":"print","content":"pub fn print() -&gt; Result(String, CounterError)\n Formats all counter metrics in the store \n as a Prometheus-compatible text string.\nSynonyms:\nthemis/counter.print\nthemis/counter print","url":"themis/counter.html#print"},{"doc":"themis/counter","title":"themis/counter","content":"","url":"themis/counter.html"},{"doc":"themis/gauge","title":"GaugeError","content":"pub type GaugeError {\n  MetricError(metric.MetricError)\n  StoreError(store.StoreError)\n  LabelError(label.LabelError)\n}\n\nMetricError(metric.MetricError)\n\n\nStoreError(store.StoreError)\n\n\nLabelError(label.LabelError)\n\n\nSynonyms:\nthemis/gauge.GaugeError\nthemis/gauge GaugeError","url":"themis/gauge.html#GaugeError"},{"doc":"themis/gauge","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n) -&gt; Result(Nil, GaugeError)\n Registers a new gauge metric to the store.\n Will return an error if the metric name is invalid\n or already used by another metric.\nSynonyms:\nthemis/gauge.new\nthemis/gauge new","url":"themis/gauge.html#new"},{"doc":"themis/gauge","title":"observe","content":"pub fn observe(\n  name name: String,\n  labels labels: Dict(String, String),\n  value value: Number,\n) -&gt; Result(Nil, GaugeError)\n Sets a gauge value for the given metric name.\n Will return an error if the name is invalid, not a registered metric\n or not of the correct metric type.\n Will return an error if any of the labels have an invalid key.\n NaN, PosInf and NegInf values are valid but not recommended.\nSynonyms:\nthemis/gauge.observe\nthemis/gauge observe","url":"themis/gauge.html#observe"},{"doc":"themis/gauge","title":"print","content":"pub fn print() -&gt; Result(String, GaugeError)\n\nSynonyms:\nthemis/gauge.print\nthemis/gauge print","url":"themis/gauge.html#print"},{"doc":"themis/gauge","title":"themis/gauge","content":"","url":"themis/gauge.html"},{"doc":"themis/histogram","title":"HistogramError","content":"pub type HistogramError {\n  MetricError(metric.MetricError)\n  StoreError(store.StoreError)\n  InvalidBucketValue\n  CannotObserveNaN\n  LabelError(label.LabelError)\n}\n\nMetricError(metric.MetricError)\n\n\nStoreError(store.StoreError)\n\n\nInvalidBucketValue\n\n\nCannotObserveNaN\n\n\nLabelError(label.LabelError)\n\n\nSynonyms:\nthemis/histogram.HistogramError\nthemis/histogram HistogramError","url":"themis/histogram.html#HistogramError"},{"doc":"themis/histogram","title":"new","content":"pub fn new(\n  name name: String,\n  description description: String,\n  buckets buckets: Set(Number),\n) -&gt; Result(Nil, HistogramError)\n Registers a new histogram metric to the store.\n The &quot;Buckets&quot; argument is a set of histogram buckets\n &quot;le&quot; values, as described here\n https://prometheus.io/docs/practices/histograms/\n Will return an error if the metric name is invalid,\n already used, or if any of the buckets have an\n invalid value (&quot;PosInf, NegInf, NaN&quot;)\nSynonyms:\nthemis/histogram.new\nthemis/histogram new","url":"themis/histogram.html#new"},{"doc":"themis/histogram","title":"observe","content":"pub fn observe(\n  name name: String,\n  labels labels: Dict(String, String),\n  value value: Number,\n) -&gt; Result(Nil, HistogramError)\n Records a datapoint into the histogram for the given metric name.\n Will return an error if the name is invalid, not a registered metric\n or not of the correct metric type.\n Will return an error if any of the labels have an invalid key.\n Will return an error if the value is number.NaN\nSynonyms:\nthemis/histogram.observe\nthemis/histogram observe","url":"themis/histogram.html#observe"},{"doc":"themis/histogram","title":"print","content":"pub fn print() -&gt; Result(String, HistogramError)\n Formats all histogram metrics in the store \n as a Prometheus-compatible text string.\nSynonyms:\nthemis/histogram.print\nthemis/histogram print","url":"themis/histogram.html#print"},{"doc":"themis/histogram","title":"themis/histogram","content":"","url":"themis/histogram.html"},{"doc":"themis","title":"ThemisError","content":"pub type ThemisError {\n  GaugeError(gauge.GaugeError)\n  HistogramError(histogram.HistogramError)\n  CounterError(counter.CounterError)\n}\n\nGaugeError(gauge.GaugeError)\n\n\nHistogramError(histogram.HistogramError)\n\n\nCounterError(counter.CounterError)\n\n\nSynonyms:\nthemis.ThemisError\nthemis ThemisError","url":"themis.html#ThemisError"},{"doc":"themis","title":"init","content":"pub fn init() -&gt; Atom\n Initializes a new empty metrics store.\nSynonyms:\nthemis.init\nthemis init","url":"themis.html#init"},{"doc":"themis","title":"print","content":"pub fn print() -&gt; Result(String, ThemisError)\n Formats all metrics in the store as a Prometheus-compatible text string.\n\n ## Examples\n\n \n let metrics_text = print(store)\n // # HELP my_metric My first gauge\n // # TYPE my_metric gauge\n // my_metric{foo=&quot;bar&quot;} 10\n // my_metric{toto=&quot;tata&quot;,wibble=&quot;wobble&quot;} +Inf\n \nSynonyms:\nthemis.print\nthemis print","url":"themis.html#print"},{"doc":"themis","title":"themis","content":"","url":"themis.html"}]);